% what happens in the log utility/no paycheck model if you shrink the time
% period to a day but inject 1/30 of one percent of the the money stock for 30 days.

clear all

%%%%% timing parameters
   
Delta = 1/360;  %% length of a period in calender time measured in fractions of a year
                %% 1/12 is monthly, 1/360 is daily etc
T     = 1.25;   %% length of a trip in years gets annual velocity of 1.6

N     = T/Delta;%% length of a trip in number of periods
N     = round(N);

h     = (1/Delta)/12; %% number of periods per month

%%%%% other parameters

beta  = 0.9999; %% annual time discount factor
mu    = 1.0001; %% annual steady-state money growth

%%%%% #1. steady state distribution of money etc

%ms    = zeros(N,1);  %% steady state money shares, ms(1) = m(0), ms(N) = m(N-1) etc
%vs    = zeros(N,1);  %% individual velocity, vs(N) = v(N-1) = 1 etc, **at annual rate**

%ms(1) = N*(1-beta^(Delta*N))/((1-((beta/mu)^(Delta*N)))/(1-(beta/mu)^Delta) - (beta^(Delta*N))*((1-mu^(-Delta*N))/(1-mu^(-Delta))));
%vs(1) = (1/Delta)*((1-(beta^Delta))/(1-beta^(Delta*(N))));

%for i=2:N,
    
%    s = i-1;
    
%    ms(i) = ms(1)*((beta/mu)^(Delta*s))*(1-(beta^(Delta*(N-s))))/(1-beta^(Delta*N));
    
%    vs(i) = (1/Delta)*((1-(beta^Delta))/(1-beta^(Delta*(N-s))));
    
%end

%vbar = (1/N)*sum(vs.*ms); %% **at annual rate**

%%%%% as a check
%     m(s) approx 2(N-s)/(N+1) for beta/mu -> 1
%     v(s) approx 1/((N-s)*Delta) for beta -> 1
%     vbar approx 2/((N+1)*Delta) for beta/mu -> 1

s = (0:1:N-1)';

ms = 2*(N-s)/(N+1);
vs = 1./((N-s)*Delta);
vbar = (1/N)*sum(vs.*ms); %% at ** annual rate **


%%%%% #2. construct shock sequence

shock = 0.01;   %% size of shock = 1% of money stock
L = N;          %% length of impulse response

mut = zeros(1,L); %% money growth shock sequence ** at annual rate **
ts  = zeros(1,L); %% period

for t = 1:L,
    
    ts(t) = t-2;
    
    if t<=1+h,
    mut(1) = 1;
    mut(2:1+h) = ((1/h)*(shock) + 1)*1; %% as Andy says, "inject 1/30 of one percent of the the money stock for 30 days"
                                         %% money stock implicitly normalized to
                                         %% one here
    else
    mut(t) = 1; %% back to steady state
    end
end

%%%%% #3. response to a shock

mst = zeros(N,L); %% matrix to store response of money share distribution
vt  = zeros(1,L); %% vector to store response of aggregate velocity

mst(:,1) = ms';   %% initially at steady state
vt(1)    = vbar;

for t = 2:L,
    
    for i=2:N,
    
        mst(i,t) = (1-vs(i-1)*Delta)*mst(i-1,t-1)/(mut(t)^Delta);
        
    end
    
    mst(1,t) = N - sum(mst(2:N,t));
    
    vt(t)    = (1/N)*sum(vs.*mst(:,t));
   
end

%%%%% log deviations as annual percentages

muhat = 100*log(mut/1)'; %% already at annual rate**
vhat  = 100*log(vt/vbar)'/Delta;

pihat = muhat(1:end-1) + diff(vhat);
phat  = cumsum(pihat);
mhat  = cumsum(muhat); 

%%%%% #4. key impact effect

display('... impact effect (at time t=0), this should be about -0.50 ...')
impact     = vhat(2)/muhat(2)       %% should be -1/2

display('... cumulative effect (at time t=30), this should be about -0.50 ...')
cumulative = vhat(31)/(100*shock)    %% should be -1/2, shock =0.01 so 100*shock = 1%

figure(2)
plot(ts,mhat,ts,vhat,ts(1:end-1),phat,ts,zeros(L,1),'k--')
axis([-1 300 -0.5 1.5])
xlabel('months after shock (each month is divided into 30 model periods)')
ylabel('deviation from steady state, annual rate')
%text(2,0.05,'1/30th of 1% of the money stock for 30 days')
legend('log(M)','log(v)','log(P)')
title('Figure II: Money up, velocity down, prices sluggish')

figure(10)
plot(ts,muhat,ts,vhat,ts,zeros(L,1),'k--')
axis([-1 300 -0.5 0.1])
xlabel('days after shock')
ylabel('deviation from steady state, annual rate')
text(2,0.05,'1/30th of 1% of the money stock for 30 days')


