%%%%% Feeding the model with money growth and output data to produce an implied velocity series

%%%%% First solve the model (with exogenous money growth and output level shocks) for the equilibrium laws of motion


clear all

%%%%% 1.		Declare structural parameters

period=12;						%% frequency of the model: 12=monthly, 4=quarterly etc
beta=(0.99)^(1/period);	    %% time discount factor	
sigma=1;						%% coefficient of relative risk aversion
N=38;							%% length of a shopping trip (no. of periods). 
								%% Also, inv(N) is measure of each type of shopper s=0,..,N-1 
gamma=0.6;						%% paycheck parameter

y_bar=1;						%% mean of income shock
rho_y=0.0;						%% persistence of income shock;
sigma_y=0.0;					%% standard deviation of income innovation

mu_bar=(1.01)^(1/period);	    %% mean of money growth factor shock
rho_mu=0.66;					%% persistence of money growth shock
sigma_mu=1;						%% standard deviation of money growth innovation   

L=12;                           %% number of lagged terms to include in estimated monthly VAR

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% 2.		Pass these parameters to the steady state function

parameters=[beta,sigma,N,gamma,y_bar,mu_bar];

[cs,ms,x]=steady_state(parameters);
			    %% cs is an N-vector of consumptions for each s=0,...,N-1
                %% ms is an N-vector of real balances for each s=1,...,N
                %% x is level of transfers. Note m(0)=m(N)+x=x               
               
%%%%% 3.		Check that we have an interior solution for the money holdings

indicator=zeros(N,1);	%% see if there are corners

for s=1:N,
   if ms(s)>=0;
      indicator(s)=1;
   else
      indicator(s)=0;
   end
end

lever=min(indicator);	%% if there are any corners, resolve using the kinky program
if lever>0

else
   [cs,ms,x,J]=resolve_kinks(parameters,indicator);
end

Ec=mean(cs);
Em=mean(ms);               
test=y_bar-Ec;
x_bar=x;               
m_bar=Em+Ec;
v_bar=y_bar/m_bar;
zs=ms(1:N-1);		%% swapped notation zs were ms!!
ZS=sum(zs);              
ws=zs/m_bar;


%%%%% 4.		Input steady state values to a function that sets up the matrices for
%				use with Harald Uhlig's toolkit

[NN,JJ,KK,AA,BB,CC,DD]=matrices_money_var(cs,zs,x_bar,m_bar,v_bar,ZS,parameters,L);

FF=zeros(N-1,N-1);
GG=zeros(N-1,N-1);
HH=zeros(N-1,N-1);
LL=zeros(N-1,2*L+1);
MM=zeros(N-1,2*L+1);

         
%%%%%	5.		Call Harald's toolkit. The equilibrium laws of motion are: 
%				xx=PPL{xx}+QQzz
%				yy=RRL{xx}+SSzz

[l_equ,m_states] = size(AA);
[l_equ,n_endog ] = size(CC);
[l_equ,k_exog  ] = size(DD);

message = '                                                                       ';
warnings = [];
options;
solve;

%%%%%   6.      Create money growth and output level shocks from data
%               Money growth shock relative to steady state is demeaned M2 growth
%               Output level shock relative to steady state is linearly detrended real PCE

%%%%% Load data

[num,txt,raw] = xlsread('data.xls'); %FEDFUNDS	GS3	M1SL	M2SL	PCE	PCEND	PCEPI	PCES

dates = 1959:1/12:2006+11/12'; T = length(dates);


ff = num(:,1)/100;
t3 = num(:,2)/100;

m1 = log(num(:,3));
m2 = log(num(:,4));

pc = log(num(:,5));
p  = log(num(:,7));
c  = pc-p;

v1 = p + c - m1;
v2 = p + c - m2;

m1c = m1 - c;
m2c = m2 - c;


iota = ones(T,1);
time = cumsum(iota);
X    =[iota,time];

fitted=X*inv(X'*X)*X'*c;

yhat=c-fitted;

m2growth=diff(m2);
meanm2growth=mean(m2growth);

muhat=(m2growth-meanm2growth);

%%%%% Normalized series

log_Md=m2-m2(1);
log_Cd=c-c(1);
log_Pd=p-p(1);
log_Vd=log_Pd+log_Cd-log_Md;

log_Md=log_Md(2:T);
log_Cd=log_Cd(2:T);
log_Pd=log_Pd(2:T);
log_Vd=log_Vd(2:T);

dates=dates(2:T);

%%%%%   7.      Feed money growth and output level shocks to the model

% drop first observation so that y(t) and y(t-1) have the same length

yhat1=yhat(2:T);
yhat2=yhat(1:T-1);

% iterate on states and endogenous variables

xx=zeros(N-1,T-1);
yy=zeros(N+2,T-1);
zz=zeros(2*L+1,T-1);

zz(1,:)=yhat1';
zz(2,:)=yhat2';
zz(L+2,:)=muhat';

%zz=[yhat1,yhat2,muhat]';

xx(:,1)=QQ*zz(:,1);
yy(:,1)=SS*zz(:,1);

for t=2:T-1,
    xx(:,t)=PP*xx(:,t-1)+QQ*zz(:,t);
    yy(:,t)=RR*xx(:,t-1)+SS*zz(:,t);
end

output=zz(1,:);
moneygrowth=zz(L+2,:);
inflation=yy(1,:);

for t=1:T-1
   balances(t)=(1/N)*sum(ws.*xx(:,t))+v_bar*output(t);
   velocity(t)=output(t)-balances(t);
   
   marginal_utility0(t)=-sigma*yy(3,t);
   
   Ereal_interest(t)=(RR(3,:)*xx(:,t)+SS(3,:)*NN*zz(:,t)-yy(3,t))*sigma;
   Einflation(t)=RR(1,:)*xx(:,t)+SS(1,:)*NN*zz(:,t);
   nominal_interest(t)=Ereal_interest(t)+Einflation(t);
   
end

log_vm=velocity';
log_vd=v2 - mean(v2); 

log_Pm=cumsum(inflation)';
log_Mm=cumsum(moneygrowth)';
log_Cm=output';



% drop first N observations to get away from initialization

T=length(log_vm);

log_vm=log_vm(N+1:T);       %% this is what comes out of the model
log_vd=log_vd(N+1:T);
dates=dates(N+1:T);

log_Mm=log_Md(N+1:T);       %% this is what we fed in exogenously
log_Cm=log_Cd(N+1:T);

log_Vm=log_vm;              %% this is what comes out of the model
log_Pm=log_Mm+log_Vm-log_Cm;%% this is implied price level



%%%%%   8. Figures and summary statistics


phi = 3^4*1600; %% 3^4*1600

[Tlog_vm,HPlog_vm]=hptrend(log_vm,phi);
[Tlog_vd,HPlog_vd]=hptrend(log_vd,phi);

figure(3)
plotyy(dates,HPlog_vd,dates,HPlog_vm)
title('Figure III: Model and data velocity')
%axis([1962 2006 -0.05 0.05])
xlabel('Month')
ylabel('Data velocity')


corr_HP=corrcoef(HPlog_vd,HPlog_vm)
std_HP=std(HPlog_vm)/std(HPlog_vd)

corr_D1 = corrcoef(diff(log_vd),diff(log_vm))
std_D1  = std(diff(log_vm))/std(diff(log_vd))

for t=13:length(log_vm),
    
    vmd12(t-12) = log_vm(t)-log_vm(t-12);
    vdd12(t-12) = log_vd(t)-log_vd(t-12);
    
end

corr_D12 = corrcoef(vmd12,vdd12)
std_D12  = std(vmd12)/std(vdd12)
