%%%%% Program to compute the non-stochastic steady state for the Baumol-Tobin type model
%	  This version of the program allows for the possibility of corners

function [cs,zs,x,J]=resolve_kinks(parameters,indicator)

%%%%%	1.		Unravel the structural parameters

beta=parameters(1);				%% time discount factor
sigma=parameters(2);			%% coefficient of relative risk aversion
N=parameters(3);				%% length of a shopping trip (no. of periods). 
								%% Also, inv(N) is measure of each type of shopper s=0,..,N-1 
gamma=parameters(4);			%% paycheck parameter
y_bar=parameters(5);			%% mean of income shock
mu_bar=parameters(6);			%% mean of money growth factor shock

%%%%%	2.		Define some transformations of these parameters

A=(mu_bar/beta)^(1/sigma);		%%	log[c(s)]=log(A)+log[c(s+1)]
w=gamma*(y_bar/mu_bar);			%%	average real paycheck in non-stochastic steady state
X=inv(mu_bar);
Y=(1-(gamma/mu_bar))*N*y_bar;

%%%%%	3.		Compute the solutions for the corner cases

%%%%%   3.01	Find J, the number of interior solutions

J=length(nonzeros(indicator));

% The first J of the consumption Euler eqs. hold and the remaining N-J are slack.
% This function should not be called unless J<N (unless indicator has some zeros).
% We make use of monotonicity; if at corner for s, then at corner for all s'>=s      

%%%%%   3.02	Set up blocks of the coefficients matrix using steady state conditions. 

%		Ordering is:

%		1) J of the Euler eqs. hold: c(s)=A*c(s+1) for s=0,...,J-1						[blocks A1,B1]
%		2) 1 equation uses the "slack" s=J bank constraint                              [blocks A2,B2]
%		3) N-J-1 equations use the "slack" bank constraints for s=J+1,...,N-1           [blocks A3,B3]

%		4) 1 equation uses the "binding" s=0 bank constraint                            [blocks A4,B4]
%		5) J-1 equations use the "binding" bank constraints for s=1,...,J-1             [blocks A5,B5]	

%		6) 1 equation uses the definition of the transfers								[blocks A6,B6]	

%		Note that there are N+J+1 equations in N+J+1 unknowns:
%
%		1) #{c(s):s=0,...,N-1}=N
%		2) #{z(s+1):s=0,...,J-1}=J
%		3) #x=1

%		Additionally, we know that z(s+1)=0 for all s=J,...,N-1. There are N-J-1 of these.
%		In the normal case, only z(N)=0; i.e., J=N-1 and there are 2N equations in 2N unknowns


%%%%% 3.1	Binding Euler Equations; Block A1,B1

%		c(s)=A*c(s+1) for s=0,...,J-1

A1=zeros(J,N+J+1);
B1=zeros(J,1);

for i=1:J,
   a=zeros(1,N+J+1);
   a(i+1)=A;
   A1(i,:)=a;
end

%%%%% 3.2	"Slack" s=J bank constraint; Block A2,B2

%		c(J)=Xz(J)+w for s=J

A2=zeros(1,N+J+1);
B2=w;

A2(N+J)=X;

%%%%% 3.3	Autarkic bank Constraints; Block A3,B3

%		c(s)=w for s=J+1,...,N-1

A3=zeros(N-J-1,N+J+1);
B3=w*ones(N-J-1,1);

%%%%% 3.4	"Binding" s=0 bank Constraint; Block A4,B4

%		m(1)=-c(0)+x+w

A4=zeros(1,N+J+1);
B4=w;

A4(1)=-1;
A4(N+J+1)=1;

%%%%% 3.5	"Binding" s=1,...,J-1 Cash Flow Constraints; Block A5,B5

%		z(s+1)=-c(s)+Xz(s)+w for s=1,...,J-1 		

A5=zeros(J-1,N+J+1);
B5=w*ones(J-1,1);

for i=1:J-1,
   a=zeros(1,N+J+1);
   a(i+1)=-1;
   a(i+N)=X;
   A5(i,:)=a;
end

%%%%% 3.6	Definition of transfers; Block A6,B6

%		x=(1-X)*[z(1)+z(2)+...+z(J)]+Y

%A6=zeros(1,N+J+1);
B6=Y;
a=zeros(1,N+J+1);

for i=1:J,
   a(i+N)=1-X;
end
A6=a;

%%%%%	4.	Collate blocks and solve for the unknowns

A=[A1;A2;A3;A4;A5;A6];
B=[B1;B2;B3;B4;B5;B6];

I=eye(N+J+1,N+J+1);
Z=(inv(I-A))*B;

cs=Z(1:N);
zs=[Z(N+1:N+J);zeros(N-J,1)];
x=Z(N+J+1);

mean(cs);

