%%%%% Program to compute the non-stochastic steady state for the Baumol-Tobin type model

function [cs,zs,x]=steady_state(parameters)

%%%%%	1.		Unravel the structural parameters

beta=parameters(1);				%% time discount factor
sigma=parameters(2);		    %% coefficient of relative risk aversion
N=parameters(3);				%% length of a shopping trip (no. of periods). 
								%% Also, inv(N) is measure of each type of shopper s=0,..,N-1 
gamma=parameters(4);			%% paycheck parameter
y_bar=parameters(5);			%% mean of income shock
mu_bar=parameters(6);			%% mean of money growth factor shock

%%%%%	2.		Define some transformations of these parameters

A=(mu_bar/beta)^(1/sigma);		%%	log[c(s)]=log(A)+log[c(s+1)]
w=gamma*(y_bar/mu_bar);			%%	average real paycheck in non-stochastic steady state
X=inv(mu_bar);
Y=(1-(gamma/mu_bar))*N*y_bar;


%%%%%   3.		Set up blocks of the coefficients matrix using steady state conditions. 

%		Ordering is:

%		1) N-1 equations use Euler Eq. c(s)=A*c(s+1) for s=0,...,N-2        [blocks A1,B1]
%		2) 1 equation uses s=N-1 bank account constraint					[blocks A2,B2]
%		3) 1 equation uses s=0 bank account constraint						[blocks A3,B3]
%		4) N-2 equations use bank account for s=1,..,N-2					[blocks A4,B4]
%		5) 1 equation uses the definition of the transfers					[blocks A5,B5]

%		Note that there are 2*N linear equations in 2*N unknowns: 
%									
%		1)	{c(s):s=0,...,N-1}
%		2)	{m(s+1):s=0,...,N-2}
%		3)	x

%		Additionaly, we know that m(N)=0 so that only N-1 Euler Eq. hold with equality

%%%%% 3.1	Block A1,B1

A1=zeros(N-1,2*N);
B1=zeros(N-1,1);

for i=1:N-1,
   a=zeros(1,2*N);
   a(i+1)=A;
   A1(i,:)=a;
end

%%%%% 3.2	Block A2,B2

A2=zeros(1,2*N);
B2=w;

A2(2*N-1)=X;

%%%%% 3.3	Block A3,B3

A3=zeros(1,2*N);
B3=w;

A3(1)=-1;
A3(2*N)=1;

%%%%% 3.4	Block A4,B4

A4=zeros(N-2,2*N);
B4=w*ones(N-2,1);

for i=1:N-2,
   a=zeros(1,2*N);
   a(i+1)=-1;
   a(i+N)=X;
   A4(i,:)=a;
end

%%%%% 3.5 Block A5,B5

%A5=zeros(1,2*N);
B5=Y;
a=zeros(1,2*N);

for i=1:N-1,
   a(i+N)=1-X;
end
A5=a;

%%%%%	4.	Collate blocks and solve for the unknowns

A=[A1;A2;A3;A4;A5];
B=[B1;B2;B3;B4;B5];

I=eye(2*N,2*N);
Z=(inv(I-A))*B;

cs=Z(1:N);
zs=[Z(N+1:2*N-1);0];
x=Z(2*N);

mean(cs);

